@@ -22,6 +22,14 @@ from pptx.spec import (
     PH_TYPE_SUBTITLE, PH_TYPE_TITLE
 )
 
+# import logging
+# log = logging.getLogger('pptx.oxml')
+
+
+# oxml-specific constants --------------
+XSD_TRUE = '1'
+
+
 # configure objectified XML parser
 fallback_lookup = objectify.ObjectifyElementClassLookup()
 element_class_lookup = etree.ElementNamespaceClassLookup(fallback_lookup)
@@ -34,7 +42,7 @@ oxml_parser.set_element_class_lookup(element_class_lookup)
 # ============================================================================
 
 def _Element(tag, nsmap=None):
-    return objectify.Element(qn(tag), nsmap=nsmap)
+    return oxml_parser.makeelement(qn(tag), nsmap=nsmap)
 
 
 def _SubElement(parent, tag, nsmap=None):
@@ -243,6 +251,43 @@ class CT_Picture(objectify.ObjectifiedElement):
         return pic
 
 
+class CT_PresetGeometry2D(objectify.ObjectifiedElement):
+    """<a:prstGeom> custom element class"""
+    @property
+    def gd(self):
+        """
+        Sequence containing the ``gd`` element children of ``<a:avLst>``
+        child element, empty if none are present.
+        """
+        try:
+            gd_elms = tuple([gd for gd in self.avLst.gd])
+        except AttributeError:
+            gd_elms = ()
+        return gd_elms
+
+    @property
+    def prst(self):
+        """Value of required ``prst`` attribute."""
+        return self.get('prst')
+
+    def rewrite_guides(self, guides):
+        """
+        Remove any ``<a:gd>`` element children of ``<a:avLst>`` and replace
+        them with ones having (name, val) in *guides*.
+        """
+        try:
+            avLst = self.avLst
+        except AttributeError:
+            avLst = _SubElement(self, 'a:avLst')
+        if hasattr(self.avLst, 'gd'):
+            for gd_elm in self.avLst.gd[:]:
+                avLst.remove(gd_elm)
+        for name, val in guides:
+            gd = _SubElement(avLst, 'a:gd')
+            gd.set('name', name)
+            gd.set('fmla', 'val %d' % val)
+
+
 class CT_Shape(objectify.ObjectifiedElement):
     """<p:sp> custom element class"""
     _autoshape_sp_tmpl = (
@@ -417,6 +462,14 @@ class CT_Shape(objectify.ObjectifiedElement):
             return None
         return prstGeom.get('prst')
 
+    @property
+    def prstGeom(self):
+        """
+        Reference to ``<a:prstGeom>`` child element or |None| if this shape
+        doesn't have one, for example, if it's a placeholder shape.
+        """
+        return _child(self.spPr, 'a:prstGeom')
+
 
 class CT_Table(objectify.ObjectifiedElement):
     """``<a:tbl>`` custom element class"""
@@ -429,6 +482,78 @@ class CT_Table(objectify.ObjectifiedElement):
         '</a:tbl>' % (nsdecls('a'), '%s')
     )
 
+    BOOLPROPS = (
+        'bandCol', 'bandRow', 'firstCol', 'firstRow', 'lastCol', 'lastRow'
+    )
+
+    def __getattr__(self, attr):
+        """
+        Implement getter side of properties. Filters ``__getattr__`` messages
+        to ObjectifiedElement base class to intercept messages intended for
+        custom property getters.
+        """
+        if attr in CT_Table.BOOLPROPS:
+            return self._get_boolean_property(attr)
+        else:
+            return super(CT_Table, self).__getattr__(attr)
+
+    def __setattr__(self, attr, value):
+        """
+        Implement setter side of properties. Filters ``__setattr__`` messages
+        to ObjectifiedElement base class to intercept messages intended for
+        custom property setters.
+        """
+        if attr in CT_Table.BOOLPROPS:
+            self._set_boolean_property(attr, value)
+        else:
+            super(CT_Table, self).__setattr__(attr, value)
+
+    def _get_boolean_property(self, propname):
+        """
+        Generalized getter for the boolean properties on the ``<a:tblPr>``
+        child element. Defaults to False if *propname* attribute is missing
+        or ``<a:tblPr>`` element itself is not present.
+        """
+        if not self.has_tblPr:
+            return False
+        return self.tblPr.get(propname) in ('1', 'true')
+
+    def _set_boolean_property(self, propname, value):
+        """
+        Generalized setter for boolean properties on the ``<a:tblPr>`` child
+        element, setting *propname* attribute appropriately based on *value*.
+        If *value* is truthy, the attribute is set to "1"; a tblPr child
+        element is added if necessary. If *value* is falsey, the *propname*
+        attribute is removed if present, allowing its default value of False
+        to be its effective value.
+        """
+        if value:
+            tblPr = self._get_or_insert_tblPr()
+            tblPr.set(propname, XSD_TRUE)
+        elif not self.has_tblPr:
+            pass
+        elif propname in self.tblPr.attrib:
+            del self.tblPr.attrib[propname]
+
+    @property
+    def has_tblPr(self):
+        """
+        True if this ``<a:tbl>`` element has a ``<a:tblPr>`` child element,
+        False otherwise.
+        """
+        try:
+            self.tblPr
+            return True
+        except AttributeError:
+            return False
+
+    def _get_or_insert_tblPr(self):
+        """Return tblPr child element, inserting a new one if not present"""
+        if not self.has_tblPr:
+            tblPr = _Element('a:tblPr')
+            self.insert(0, tblPr)
+        return self.tblPr
+
     @staticmethod
     def new_tbl(rows, cols, width, height, tableStyleId=None):
         """Return a new ``<p:tbl>`` element tree"""
@@ -482,6 +607,113 @@ class CT_TableCell(objectify.ObjectifiedElement):
         objectify.deannotate(tc, cleanup_namespaces=True)
         return tc
 
+    @property
+    def anchor(self):
+        """
+        String held in ``anchor`` attribute of ``<a:tcPr>`` child element of
+        this ``<a:tc>`` element.
+        """
+        if not hasattr(self, 'tcPr'):
+            return None
+        return self.tcPr.get('anchor')
+
+    def _set_anchor(self, anchor):
+        """
+        Set value of anchor attribute on ``<a:tcPr>`` child element
+        """
+        if anchor is None:
+            return self._clear_anchor()
+        if not hasattr(self, 'tcPr'):
+            tcPr = _Element('a:tcPr')
+            idx = 1 if hasattr(self, 'txBody') else 0
+            self.insert(idx, tcPr)
+        self.tcPr.set('anchor', anchor)
+
+    def _clear_anchor(self):
+        """
+        Remove anchor attribute from ``<a:tcPr>`` if it exists and remove
+        ``<a:tcPr>`` element if it then has no attributes.
+        """
+        if not hasattr(self, 'tcPr'):
+            return
+        if 'anchor' in self.tcPr.attrib:
+            del self.tcPr.attrib['anchor']
+        if len(self.tcPr.attrib) == 0:
+            self.remove(self.tcPr)
+
+    def __get_marX(self, attr_name, default):
+        """generalized method to get margin values"""
+        if not hasattr(self, 'tcPr'):
+            return default
+        return int(self.tcPr.get(attr_name, default))
+
+    @property
+    def marT(self):
+        """
+        Read/write integer top margin value represented in ``marT`` attribute
+        of the ``<a:tcPr>`` child element of this ``<a:tc>`` element. If the
+        attribute is not present, the default value ``45720`` (0.05 inches)
+        is returned for top and bottom; ``91440`` (0.10 inches) is the
+        default for left and right. Assigning |None| to any ``marX``
+        property clears that attribute from the element, effectively setting
+        it to the default value.
+        """
+        return self.__get_marX('marT', 45720)
+
+    @property
+    def marR(self):
+        """right margin value represented in ``marR`` attribute"""
+        return self.__get_marX('marR', 91440)
+
+    @property
+    def marB(self):
+        """bottom margin value represented in ``marB`` attribute"""
+        return self.__get_marX('marB', 45720)
+
+    @property
+    def marL(self):
+        """left margin value represented in ``marL`` attribute"""
+        return self.__get_marX('marL', 91440)
+
+    def _set_marX(self, marX, value):
+        """
+        Set value of marX attribute on ``<a:tcPr>`` child element. If *marX*
+        is |None|, the marX attribute is removed and the ``<a:tcPr>`` element
+        is removed if it then has no attributes.
+        """
+        if value is None:
+            return self.__clear_marX(marX)
+        if not hasattr(self, 'tcPr'):
+            tcPr = _Element('a:tcPr')
+            idx = 1 if hasattr(self, 'txBody') else 0
+            self.insert(idx, tcPr)
+        self.tcPr.set(marX, str(value))
+
+    def __clear_marX(self, marX):
+        """
+        Remove marX attribute from ``<a:tcPr>`` if it exists and remove
+        ``<a:tcPr>`` element if it then has no attributes.
+        """
+        if not hasattr(self, 'tcPr'):
+            return
+        if marX in self.tcPr.attrib:
+            del self.tcPr.attrib[marX]
+        if len(self.tcPr.attrib) == 0:
+            self.remove(self.tcPr)
+
+    def __setattr__(self, attr, value):
+        """
+        This hack is needed to make setter side of properties work,
+        overrides ``__setattr__`` defined in ObjectifiedElement super class
+        just enough to route messages intended for custom property setters.
+        """
+        if attr == 'anchor':
+            self._set_anchor(value)
+        elif attr in ('marT', 'marR', 'marB', 'marL'):
+            self._set_marX(attr, value)
+        else:
+            super(CT_TableCell, self).__setattr__(attr, value)
+
 
 class CT_TextBody(objectify.ObjectifiedElement):
     """<p:txBody> custom element class"""
@@ -502,7 +734,54 @@ class CT_TextBody(objectify.ObjectifiedElement):
         return txBody
 
 
+class CT_TextParagraph(objectify.ObjectifiedElement):
+    """<a:p> custom element class"""
+    def get_algn(self):
+        """
+        Paragraph horizontal alignment value, like ``TAT.CENTER``. Value of
+        algn attribute on <a:pPr> child element
+        """
+        if not hasattr(self, 'pPr'):
+            return None
+        return self.pPr.get('algn')
+
+    def set_algn(self, value):
+        """
+        Set value of algn attribute on <a:pPr> child element
+        """
+        if value is None:
+            return self._clear_algn()
+        if not hasattr(self, 'pPr'):
+            pPr = _Element('a:pPr')
+            self.insert(0, pPr)
+        self.pPr.set('algn', value)
+
+    def _clear_algn(self):
+        """
+        Remove algn attribute from ``<a:pPr>`` if it exists and remove
+        ``<a:pPr>`` element if it then has no attributes.
+        """
+        if not hasattr(self, 'pPr'):
+            return
+        if 'algn' in self.pPr.attrib:
+            del self.pPr.attrib['algn']
+        if len(self.pPr.attrib) == 0:
+            self.remove(self.pPr)
+
+    # def __setattr__(self, attr, value):
+    #     """
+    #     This hack is needed to override ``__setattr__`` defined in
+    #     ObjectifiedElement super class.
+    #     """
+    #     if attr == 'algn':
+    #         self._set_algn(value)
+    #     else:
+    #         super(CT_TextParagraph, self).__setattr__(attr, value)
+
+
 a_namespace = element_class_lookup.get_namespace(nsmap['a'])
+a_namespace['p'] = CT_TextParagraph
+a_namespace['prstGeom'] = CT_PresetGeometry2D
 a_namespace['tbl'] = CT_Table
 a_namespace['tc'] = CT_TableCell
 
