@@ -35,16 +35,8 @@ from pptx.exceptions import (
 from pptx.spec import qtag
 from pptx.spec import PTS_HASRELS_NEVER, PTS_HASRELS_OPTIONAL
 
-import logging
-log = logging.getLogger('pptx.packaging')
-log.setLevel(logging.DEBUG)
-# log.setLevel(logging.INFO)
-ch = logging.StreamHandler()
-ch.setLevel(logging.DEBUG)
-formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s'
-                              ' - %(message)s')
-ch.setFormatter(formatter)
-log.addHandler(ch)
+# import logging
+# log = logging.getLogger('pptx.packaging')
 
 PKG_BASE_URI = '/'
 
